"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var use_event_listener_1 = __importDefault(require("@utilityjs/use-event-listener"));
var MODIFIERS = {
    Alt: "Alt",
    Control: "Control",
    Meta: "Meta",
    Shift: "Shift"
};
var EVENT_MODIFIERS = {
    Alt: "altKey",
    Control: "ctrlKey",
    Meta: "metaKey",
    Shift: "shiftKey"
};
var KEYS_KEYCODE = {
    Space: "Space",
    ";": "Semicolon",
    "=": "Equal",
    ",": "Comma",
    "-": "Minus",
    ".": "Period",
    "/": "Slash",
    "`": "Backquote",
    "[": "BracketLeft",
    "\\": "Backslash",
    "]": "BracketRight",
    "'": "Quote",
    "1": "Digit1",
    "2": "Digit2",
    "3": "Digit3",
    "4": "Digit4",
    "5": "Digit5",
    "6": "Digit6",
    "7": "Digit7",
    "8": "Digit8",
    "9": "Digit9",
    "0": "Digit0",
    a: "KeyA",
    b: "KeyB",
    c: "KeyC",
    d: "KeyD",
    e: "KeyE",
    f: "KeyF",
    g: "KeyG",
    h: "KeyH",
    i: "KeyI",
    j: "KeyJ",
    k: "KeyK",
    l: "KeyL",
    m: "KeyM",
    n: "KeyN",
    o: "KeyO",
    p: "KeyP",
    q: "KeyQ",
    r: "KeyR",
    s: "KeyS",
    t: "KeyT",
    u: "KeyU",
    v: "KeyV",
    w: "KeyW",
    x: "KeyX",
    y: "KeyY",
    z: "KeyZ"
};
var KEYS_KEYNAME = {
    Backspace: "Backspace",
    Tab: "Tab",
    Enter: "Enter",
    Shift: "Shift",
    Control: "Control",
    Alt: "Alt",
    CapsLock: "CapsLock",
    Escape: "Escape",
    End: "End",
    Home: "Home",
    ArrowLeft: "ArrowLeft",
    ArrowUp: "ArrowUp",
    ArrowRight: "ArrowRight",
    ArrowDown: "ArrowDown",
    Insert: "Insert",
    Delete: "Delete",
    Meta: "Meta",
    NumLock: "NumLock",
    F1: "F1",
    F2: "F2",
    F3: "F3",
    F4: "F4",
    F5: "F5",
    F6: "F6",
    F7: "F7",
    F8: "F8",
    F9: "F9",
    F10: "F10",
    F11: "F11",
    F12: "F12"
};
var keybindHandler = function (event, keys, callback) {
    if (event.defaultPrevented)
        return;
    var hasModifier = keys.some(function (key) { return !!MODIFIERS[key]; }) || false;
    var eventModifiers = Object.keys(EVENT_MODIFIERS)
        .map(function (key) {
        var eventMod = EVENT_MODIFIERS[key];
        if (event[eventMod]) {
            if (!hasModifier)
                return null;
            return eventMod;
        }
        return null;
    })
        .filter(Boolean);
    var bail = false;
    keys.forEach(function (key) {
        if (bail)
            return;
        var isModifier = !!MODIFIERS[key];
        var eventModKey = EVENT_MODIFIERS[key];
        if (isModifier) {
            var modKeyIndex = eventModifiers.indexOf(eventModKey);
            if (modKeyIndex > -1) {
                eventModifiers.splice(modKeyIndex, 1);
                return;
            }
            bail = true;
            return;
        }
        if ((KEYS_KEYNAME[key] &&
            event.key !== KEYS_KEYNAME[key]) ||
            (KEYS_KEYCODE[key] &&
                event.code !== KEYS_KEYCODE[key])) {
            bail = true;
        }
    });
    if (bail)
        return;
    if (eventModifiers.length > 0)
        return;
    callback(event);
};
var useKeybind = function (target, bindings) {
    (0, use_event_listener_1.default)({
        target: target,
        eventType: "keydown",
        handler: function (event) {
            bindings.forEach(function (binding) {
                keybindHandler(event, binding.keys, binding.callback);
            });
        }
    });
};
exports.default = useKeybind;
